@@ -25,7 +25,6 @@
 #include <string.h>
 #include <unistd.h>
 
-
 /**
  * progress.h - only for progress bar
  */
@@ -125,6 +124,9 @@ int main(int argc, char **argv){
      * restore mode, source is image file/stdin and target is device
      * dd mode, source is device and target is device !!not complete
      */
+#ifdef _FILE_OFFSET_BITS
+    log_mesg(1, 0, 0, debug, "enable _FILE_OFFSET_BITS %i\n", _FILE_OFFSET_BITS);
+#endif
     source = opt.source;
     target = opt.target;
     dfr = open_source(source, &opt);
@@ -169,7 +171,8 @@ int main(int argc, char **argv){
 	log_mesg(1, 0, 0, debug, "done\n");
 
 	needed_size = (unsigned long long)(((image_hdr.block_size+sizeof(unsigned long))*image_hdr.usedblocks)+sizeof(image_hdr)+sizeof(char)*image_hdr.totalblock);
-	check_free_space(&dfw, needed_size);
+	if (opt.check)
+	    check_free_space(&dfw, needed_size);
 
 	log_mesg(2, 0, 0, debug, "check main bitmap pointer %i\n", bitmap);
 
@@ -222,7 +225,8 @@ int main(int argc, char **argv){
 	get_image_bitmap(&dfr, opt, image_hdr, bitmap);
 
 	/// check the dest partition size.
-	check_size(&dfw, image_hdr.device_size);
+	if((opt.check) && (check_size(&dfw, image_hdr.device_size) != 0))
+            log_mesg(0, 0, 1, debug, "Can't get device size, use option -C to disable size checkinh.\n");
 
 	log_mesg(2, 0, 0, debug, "check main bitmap pointer %i\n", bitmap);
     } else if (opt.dd){
@@ -249,6 +253,10 @@ int main(int argc, char **argv){
 	/// read and check bitmap from partition
 	log_mesg(1, 0, 0, debug, "Calculating bitmap ...\n");
 	readbitmap(source, image_hdr, bitmap);
+	
+	/// check the dest partition size.
+	if((opt.check) && (check_size(&dfw, image_hdr.device_size) != 0))
+            log_mesg(0, 0, 1, debug, "Can't get device size, use option -C to disable size checkinh.\n");
 
 	log_mesg(2, 0, 0, debug, "check main bitmap pointer %i\n", bitmap);
     }
@@ -306,15 +314,21 @@ int main(int argc, char **argv){
 	    if((image_hdr.totalblock - 1 ) == block_id) 
 		done = 1;
 
+#ifdef _FILE_OFFSET_BITS
+	    if(copied == image_hdr.usedblocks) 
+		done = 1;
+#endif
 	    if (bitmap[block_id] == 1){
 		/// if the block is used
 		log_mesg(1, 0, 0, debug, "block_id=%lli, ",block_id);
 		log_mesg(1, 0, 0, debug, "bitmap=%i, ",bitmap[block_id]);
 
 		offset = (off_t)(block_id * image_hdr.block_size);
-		//sf = lseek(dfr, offset, SEEK_SET);
-                //if (sf == (off_t)-1)
-                //    log_mesg(0, 1, 1, debug, "seek error %lli errno=%i\n", (long long)offset, (int)errno);
+#ifdef _FILE_OFFSET_BITS
+		sf = lseek(dfr, offset, SEEK_SET);
+		if (sf == -1)
+		    log_mesg(0, 1, 1, debug, "source seek error = %lli, ",sf);
+#endif
         	buffer = (char*)malloc(image_hdr.block_size); ///alloc a memory to copy data
                 if(buffer == NULL){
                     log_mesg(0, 1, 1, debug, "%s, %i, ERROR:%s", __func__, __LINE__, strerror(errno));
@@ -367,6 +381,7 @@ int main(int argc, char **argv){
 		    log_mesg(0, 1, 1, debug, "read and write different\n");
 		log_mesg(1, 0, 0, debug, "end\n");
             } else {
+#ifndef _FILE_OFFSET_BITS
 		/// if the block is not used, I just skip it.
 		log_mesg(2, 0, 0, debug, "block_id=%lli, ",block_id);
         	sf = lseek(dfr, image_hdr.block_size, SEEK_CUR);
@@ -380,6 +395,7 @@ int main(int argc, char **argv){
 		    s_count = 0;
 		}
 		log_mesg(2, 0, 0, debug, "end\n");
+#endif
 	    }
         } /// end of for    
 	sync_data(dfw, &opt);	
@@ -418,15 +434,21 @@ int main(int argc, char **argv){
 	    if((block_id + 1) == image_hdr.totalblock) 
 		done = 1;
 
+#ifdef _FILE_OFFSET_BITS
+	    if(copied == image_hdr.usedblocks) 
+		done = 1;
+#endif
 	    if (bitmap[block_id] == 1){ 
 		/// The block is used
 		log_mesg(2, 0, 0, debug, "block_id=%lli, ",block_id);
 		log_mesg(1, 0, 0, debug, "bitmap=%i, ",bitmap[block_id]);
 
 		offset = (off_t)(block_id * image_hdr.block_size);
-		//sf = lseek(dfw, offset, SEEK_SET);
-		//if (sf == (off_t)-1)
-		//    log_mesg(0, 1, 1, debug, "seek error %lli errno=%i\n", (long long)offset, (int)errno);
+#ifdef _FILE_OFFSET_BITS
+		sf = lseek(dfw, offset, SEEK_SET);
+		if (sf == -1)
+		    log_mesg(0, 1, 1, debug, "target seek error = %lli, ",sf);
+#endif
 		buffer = (char*)malloc(image_hdr.block_size); ///alloc a memory to copy data
                 if(buffer == NULL){
                     log_mesg(0, 1, 1, debug, "%s, %i, ERROR:%s", __func__, __LINE__, strerror(errno));
@@ -467,7 +489,7 @@ int main(int argc, char **argv){
 		//	log_mesg(0, 1, 1, debug, "read and write different\n");
 		log_mesg(1, 0, 0, debug, "end\n");
 	    } else {
-
+#ifndef _FILE_OFFSET_BITS
 		/// if the block is not used, I just skip it.
 		log_mesg(2, 0, 0, debug, "block_id=%lli, ",block_id);
 		sf = lseek(dfw, image_hdr.block_size, SEEK_CUR);
@@ -480,6 +502,7 @@ int main(int argc, char **argv){
 		    s_count = 0;
 		}
 		log_mesg(2, 0, 0, debug, "end\n");
+#endif
 	    }
 
     	} // end of for
@@ -500,12 +523,24 @@ int main(int argc, char **argv){
 	    if((image_hdr.totalblock - 1 ) == block_id)
 		done = 1;
 
+#ifdef _FILE_OFFSET_BITS
+	    if(copied == image_hdr.usedblocks) 
+		done = 1;
+#endif
 	    if (bitmap[block_id] == 1){
 		/// if the block is used
 
 		log_mesg(1, 0, 0, debug, "block_id=%lli, ",block_id);
 		log_mesg(1, 0, 0, debug, "bitmap=%i, ",bitmap[block_id]);
 		offset = (off_t)(block_id * image_hdr.block_size);
+#ifdef _FILE_OFFSET_BITS
+		sf = lseek(dfr, offset, SEEK_SET);
+		if (sf == -1)
+		    log_mesg(0, 1, 1, debug, "source seek error = %lli, ",sf);
+		sf = lseek(dfw, offset, SEEK_SET);
+		if (sf == -1)
+		    log_mesg(0, 1, 1, debug, "target seek error = %lli, ",sf);
+#endif
 		buffer = (char*)malloc(image_hdr.block_size); ///alloc a memory to copy data
                 if(buffer == NULL){
                     log_mesg(0, 1, 1, debug, "%s, %i, ERROR:%s", __func__, __LINE__, strerror(errno));
@@ -543,6 +578,7 @@ int main(int argc, char **argv){
 		    log_mesg(0, 1, 1, debug, "read and write different\n");
 		log_mesg(1, 0, 0, debug, "end\n");
 	    } else {
+#ifndef _FILE_OFFSET_BITS
 		/// if the block is not used, I just skip it.
 		log_mesg(2, 0, 0, debug, "block_id=%lli, ",block_id);
 		sf = lseek(dfr, image_hdr.block_size, SEEK_CUR);
@@ -558,6 +594,7 @@ int main(int argc, char **argv){
 		}
 
 		log_mesg(2, 0, 0, debug, "end\n");
+#endif
 	    }
 	} /// end of for
 	sync_data(dfw, &opt);
