@@ -51,7 +51,7 @@ FILE* msg = NULL;
  */
 extern void usage(void)
 {
-    fprintf(stderr, "%s v%s (%s) http://partclone.sourceforge.net\nUsage: %s [OPTIONS]\n"
+    fprintf(stderr, "%s v%s (%s) partclone.sourceforge.net partclone.nchc.org.tw\nUsage: %s [OPTIONS]\n"
         "    Efficiently clone to a image, device or standard output.\n"
         "\n"
         "    -o,  --output FILE      Output FILE\n"
@@ -62,6 +62,7 @@ extern void usage(void)
 	"    -b,  --dd-mode          Save to sector-to-sector format\n"
         "    -dX, --debug=X          Set the debug level to X = [0|1|2]\n"
         "    -R,  --rescue           Continue after disk read errors\n"
+        "    -C,  --no_check         Don't check device size and free space\n"
         "    -h,  --help             Display this help\n"
     , EXECNAME, VERSION, svn_version, EXECNAME);
     exit(0);
@@ -69,7 +70,7 @@ extern void usage(void)
 
 extern void parse_options(int argc, char **argv, cmd_opt* opt)
 {
-    static const char *sopt = "-hd::cbro:O:s:R";
+    static const char *sopt = "-hd::cbro:O:s:RC";
     static const struct option lopt[] = {
         { "help",		no_argument,	    NULL,   'h' },
         { "output",		required_argument,  NULL,   'o' },
@@ -80,6 +81,7 @@ extern void parse_options(int argc, char **argv, cmd_opt* opt)
         { "dd-mode",		no_argument,	    NULL,   'b' },
         { "debug",		optional_argument,  NULL,   'd' },
         { "rescue",		no_argument,	    NULL,   'R' },
+        { "check",		no_argument,	    NULL,   'C' },
         { NULL,			0,		    NULL,    0  }
     };
 
@@ -87,6 +89,8 @@ extern void parse_options(int argc, char **argv, cmd_opt* opt)
     int mode = 0;
     memset(opt, 0, sizeof(cmd_opt));
     opt->debug = 0;
+    opt->rescue = 0;
+    opt->check = 1;
 
     while ((c = getopt_long(argc, argv, sopt, lopt, NULL)) != (char)-1) {
             switch (c) {
@@ -125,6 +129,9 @@ extern void parse_options(int argc, char **argv, cmd_opt* opt)
 	    case 'R':
 		    opt->rescue++;
 		    break;
+	    case 'C':
+		    opt->check = 0;
+		    break;
             default:
                     fprintf(stderr, "Unknown option '%s'.\n", argv[optind-1]);
                     usage();
@@ -229,6 +236,7 @@ extern void close_log(){
 extern void restore_image_hdr(int* ret, cmd_opt* opt, image_head* image_hdr){
     int r_size;
     char* buffer;
+    unsigned long long dev_size;
     int debug = opt->debug;
 
     buffer = (char*)malloc(sizeof(image_head));
@@ -238,20 +246,40 @@ extern void restore_image_hdr(int* ret, cmd_opt* opt, image_head* image_hdr){
         log_mesg(0, 1, 1, debug, "read image_hdr error\n");
     memcpy(image_hdr, buffer, sizeof(image_head));
     free(buffer);
+    dev_size = (unsigned long long)(image_hdr->totalblock * image_hdr->block_size);
+    if (image_hdr->device_size != dev_size)
+	image_hdr->device_size = dev_size;
 }
 
 /// check partition size
-extern void check_size(int* ret, unsigned long long size){
+extern int check_size(int* ret, unsigned long long size){
 
     unsigned long long dest_size;
-    long dest_block;
+    unsigned long dest_block;
     int debug = 1;
 
-    if (ioctl(*ret, BLKGETSIZE, &dest_block) >= 0)
-	dest_size = (unsigned long long)(512*dest_block);
-
+#ifdef BLKGETSIZE64
+    if (ioctl(*ret, BLKGETSIZE64, &dest_size) < 0) {
+        log_mesg(0, 0, 0, debug, "get device size error\n");
+    }
+    log_mesg(0, 0, 0, debug, "Device - Target size: %lliMB  Original size: %lliMB\n", print_size(dest_size, MBYTE), print_size(size, MBYTE));
     if (dest_size < size)
-	log_mesg(0, 1, 1, debug, "The dest partition size is small than original partition.(%lliMB < %lliMB)\n", print_size(dest_size, MBYTE), print_size(size, MBYTE));
+	log_mesg(0, 1, 1, debug, "The dest partition size is smaller than original partition. (Target: %lliMB < Original: %lliMB)\n", print_size(dest_size, MBYTE), print_size(size, MBYTE));
+    return 1;
+#endif
+
+#ifdef BLKGETSIZE
+    if (ioctl(*ret, BLKGETSIZE, &dest_block) >= 0) {
+            dest_size = (unsigned long long)(dest_block * 512);
+    }
+    log_mesg(0, 0, 0, debug, "Device - Target size: %lliMB  Original size: %lliMB\n", print_size(dest_size, MBYTE), print_size(size, MBYTE));
+    if (dest_size < size)
+	log_mesg(0, 1, 1, debug, "The dest partition size is smaller than original partition. (Target: %lliMB < Original: %lliMB)\n", print_size(dest_size, MBYTE), print_size(size, MBYTE));
+    return 1;
+#endif
+
+    return 0;
+
 }
 
 /// chech free space 
@@ -487,7 +515,7 @@ extern int io_all(int *fd, char *buf, int count, int do_write, cmd_opt* opt)
 
 extern void sync_data(int fd, cmd_opt* opt)
 {
-    log_mesg(0, 0, 1, opt->debug, "Syncing ...");
+    log_mesg(0, 0, 1, opt->debug, "Syncing... ");
 	if (fsync(fd) && errno != EINVAL)
 	log_mesg(0, 1, 1, opt->debug, "fsync error: errno = %i\n", errno);
     log_mesg(0, 0, 1, opt->debug, "OK!\n");
@@ -566,6 +594,7 @@ extern void print_opt(cmd_opt opt){
     log_mesg(1, 0, 0, debug, "TARGET: %s\n", opt.target);
     log_mesg(1, 0, 0, debug, "OVERWRITE: %i\n", opt.overwrite);
     log_mesg(1, 0, 0, debug, "RESCUE: %i\n", opt.rescue);
+    log_mesg(1, 0, 0, debug, "CHECK: %i\n", opt.check);
 
 }
 
@@ -582,7 +611,7 @@ extern void print_image_hdr_info(image_head image_hdr, cmd_opt opt){
 	bindtextdomain(PACKAGE, LOCALEDIR);
 	textdomain(PACKAGE);
 	//log_mesg(0, 0, 0, "%s v%s \n", EXEC_NAME, VERSION);
-	log_mesg(0, 0, 1, debug, _("Partclone v%s (%s) http://partclone.sourceforge.net\n"), VERSION, svn_version);
+	log_mesg(0, 0, 1, debug, _("Partclone v%s (%s) partclone.sourceforge.net partclone.nchc.org.tw\n"), VERSION, svn_version);
     if (opt.clone)
 		log_mesg(0, 0, 1, debug, _("Starting clone device (%s) to image (%s)\n"), opt.source, opt.target);	
 	else if(opt.restore)
@@ -607,11 +636,11 @@ extern void print_finish_info(cmd_opt opt){
     bindtextdomain(PACKAGE, LOCALEDIR);
     textdomain(PACKAGE);
     if (opt.clone)
-	log_mesg(0, 0, 1, debug, _("Partclone successful clone device (%s) to image (%s)\n"), opt.source, opt.target);	
+	log_mesg(0, 0, 1, debug, _("Partclone successfully clone the device (%s) to the image (%s)\n"), opt.source, opt.target);	
     else if(opt.restore)
-	log_mesg(0, 0, 1, debug, _("Partclone successful restore image (%s) to device (%s)\n"), opt.source, opt.target);
+	log_mesg(0, 0, 1, debug, _("Partclone successfully restore the image (%s) to the device (%s)\n"), opt.source, opt.target);
     else if(opt.dd)
-	log_mesg(0, 0, 1, debug, _("Partclone successful clone device (%s) to device (%s)\n"), opt.source, opt.target);
+	log_mesg(0, 0, 1, debug, _("Partclone successfully clone the device (%s) to the device (%s)\n"), opt.source, opt.target);
 	    
 }
 
